﻿using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Dienstfahrten.Klassen;
using Microsoft.SharePoint;
using Dienstfahrten.Logging;
using Microsoft.SharePoint.WebControls;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;

namespace Dienstfahrten.Dienstfahrten
{
    public partial class DienstfahrtenUserControl : UserControl
    {
        SPWeb currentWeb;
        Log log;
        private int _valiErrors;

        protected void Page_Load(object sender, EventArgs e)
        {
            currentWeb = SPContext.Current.Web;
            log = new Log();

            init();
            

            if (!Page.IsPostBack)
            {
                loadData(currentWeb);                    
            }
        }


        public void loadData(SPWeb currentWeb) 
        {
            //Dienstfahrten
            //Fix für das Layout beim DateTimeControl. Tritt bei Managed Path auf ( http://stackoverflow.com/questions/4243408/sharepoint-2010-datetimecontrol-styling-issues )
            //inputDatum.DatePickerFrameUrl = SPContext.Current.Web.ServerRelativeUrl + "/_layouts/iframe.aspx";
            Dienstfahrt df = new Dienstfahrt(currentWeb);

            //Favoritenliste
            DienstfahrtListe dlf = new DienstfahrtListe(currentWeb);
            grdFavoriten.DataSource = dlf.getFavorites(string.Empty);
            grdFavoriten.DataBind();

            //nicht versandte Dienstfahrten dieses Jahres
            DienstfahrtListe dfl = new DienstfahrtListe(currentWeb);
            List<Object> dflList = dfl.getListbyAktuellesJahr(false, false, string.Empty);
            grdDienstfahrtenAktuell.DataSource = dflList;
            grdDienstfahrtenAktuell.DataBind();
            lbldfSummierteStrecke.Text = helper.berechneSummeDienstfahrtenKM(dflList);

            //Konfiguration
            if (dflList.Count > 0)
            {
                Konfiguration konf = new Konfiguration(currentWeb);
                konf.getByUser(currentWeb.CurrentUser);

                btndfVerschicken.Enabled = true;
                if (konf.Empfaenger != null)
                    btndfVerschicken.Text = helper.GetLocalizedString("dl_SendeDaten") + " " + konf.Empfaenger.Name;
                //btnVerschicken.Text = DfMultiLanguabe.GetString("dl_SendeDaten") + " " + konf.Empfaenger.Name;
                else btndfVerschicken.Text = helper.GetLocalizedString("dl_keinEmpfaenger");
            }
            else btndfVerschicken.Enabled = false;
        }

        /// <summary>
        /// Initialisiere Sichtbarkeint der Info/Fehler-Ausgaben
        /// </summary>
        private void init()
        {
            //Dienstfahrt Voreinstellungen
            lblinputDatumError.Visible = false;
            lblsaveDienstfahrtInfo.Visible = false;
            lblinputVonError.Visible = false;
            lblinputNachError.Visible = false;
            lblinputStreckeEinfachError.Visible = false;
            inputDatum.SelectedDate = DateTime.Now;
            lbldfbtnVerschickenInfo.Visible = false;
        }


        /// <summary>
        /// Favorit übernehmen/ Favorit löschen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnFavorit_click(object sender, CommandEventArgs e)
        {
            if (e.CommandArgument.ToString() != string.Empty)
            {
                try
                {
                    int id = int.Parse(e.CommandArgument.ToString());
                    Dienstfahrt df = new Dienstfahrt(currentWeb);

                    if (e.CommandName == "btnGetDienstfahrt")
                    {
                        df = (Dienstfahrt)df.getById(id);

                        inputVon.Text = df.Von;
                        inputNach.Text = df.Nach;
                        inputStreckeEinfach.Text = df.Einfach.ToString();
                        cbxGesamt.Checked = (df.Gesamt > 0);
                    }

                    if (e.CommandName == "btnDelFavorit")
                    {
                        df.deleteById(id);
                    }

                    loadData(currentWeb);
                }
                catch (Exception ex)
                {
                    log.Error(ex);
                }

            }
        }


        /// <summary>
        /// Favorit speichern
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnSaveFavorit_click(object sender, EventArgs e)
        {
            saveDienstfahrt(true);

            inputDatum.SelectedDate = DateTime.Now;
            inputVon.Text = string.Empty;
            inputNach.Text = string.Empty;
            inputStreckeEinfach.Text = string.Empty;
            cbxGesamt.Checked = false;
        }

        /// <summary>
        /// Dienstfahrt speichern
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnSaveDienstfahrt_click(object sender, EventArgs e)
        {
            saveDienstfahrt(false);
            //leere Felder
            //inputDatum.Text = string.Empty;
            inputDatum.SelectedDate = DateTime.Now;
            inputVon.Text = string.Empty;
            inputNach.Text = string.Empty;
            inputStreckeEinfach.Text = string.Empty;
            cbxGesamt.Checked = false;
        }

        /// <summary>
        /// Speichere eine Dienstfahrt
        /// </summary>
        /// <returns></returns>
        private void saveDienstfahrt(bool fav)
        {
            #region Validierung

            _valiErrors = 0;
            DateTime datum = new DateTime();
            if (!inputDatum.IsDateEmpty)
                datum = inputDatum.SelectedDate;
            else
            {
                lblinputDatumError.Visible = true;
                _valiErrors++;
            }

            string von = helper.validateTextBoxString(lblinputVonError, inputVon, ref _valiErrors);
            string nach = helper.validateTextBoxString(lblinputNachError, inputNach, ref _valiErrors);
            int einfach = helper.validateTextBoxInt(lblinputStreckeEinfachError, inputStreckeEinfach, ref _valiErrors);
            int gesamt = einfach;
            if (cbxGesamt.Checked) gesamt = einfach * 2;

            if (_valiErrors != 0) return;
            #endregion

            Dienstfahrt df = new Dienstfahrt(currentWeb);
            df.setDienstfahrtValues(datum, von, nach, einfach, gesamt, fav, false);
            if (df.saveToList())
                lblsaveDienstfahrtInfo.Text = helper.GetLocalizedString("d_SpeichernErfolg");
            else
                lblsaveDienstfahrtInfo.Text = helper.GetLocalizedString("d_SpeichernFehler");

            lblsaveDienstfahrtInfo.Visible = true;

            loadData(currentWeb);

        }

        /// <summary>
        /// Aktuelle unverschickte Daten verschicken.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnVerschicken_click(object sender, EventArgs e)
        {
            Konfiguration konf = new Konfiguration(currentWeb);
            Konfiguration test = (Konfiguration)konf.getByUser(currentWeb.CurrentUser);

            DienstfahrtListe dl = new DienstfahrtListe(currentWeb);
            List<Object> dienstfahrten = new List<Object>();
            dienstfahrten = dl.getListbyAktuellesJahr(false, false, null);

            bool result = false;
            if (test != null && dl != null)
            {
                Mail mail = new Mail(currentWeb, test, dienstfahrten);
                result = mail.createAndSendMail();
            }
            else result = false;

            lbldfbtnVerschickenInfo.Visible = true;

            if (result)
            {
                foreach (Dienstfahrt item in dienstfahrten)
                {
                    item.saveVerschicktToList(item.ID);
                }
                lbldfbtnVerschickenInfo.Text = helper.GetLocalizedString("dl_DatenGesendetErfolg");
            }
            else
            {
                lbldfbtnVerschickenInfo.Text = helper.GetLocalizedString("dl_keinEmpfaenger");
            }

            loadData(currentWeb);
        }

        /// <summary>
        /// Dienstfahrt löschen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btndfDeleteDienstfahrt_click(object sender, CommandEventArgs e)
        {
            if (e.CommandName == "btndfDeleteDienstfahrt" && e.CommandArgument.ToString() != string.Empty)
            {
                try
                {
                    int id = int.Parse(e.CommandArgument.ToString());
                    Dienstfahrt df = new Dienstfahrt(currentWeb);
                    df.deleteById(id);

                    loadData(currentWeb);
                }
                catch (Exception ex)
                {
                    log.Error(ex);
                }
                
            }
        }

        //##############################
        //Gridview Einstellungen
        //##############################
        protected void grdViewFavoriten_PageIndexChanging(object sender, GridViewPageEventArgs e)
        {
            grdFavoriten.PageIndex = e.NewPageIndex;
        }

        protected void gridViewFavoriten_Sorting(object sender, GridViewSortEventArgs e)
        {
            string lastExpression = string.Empty;
            if (ViewState["SortExpression"] != null)
                lastExpression = ViewState["SortExpression"].ToString();

            SortDirection lastDirection = SortDirection.Ascending;
            if (ViewState["SortDirection"] != null)
                lastDirection = (SortDirection)ViewState["SortDirection"];

            SortDirection newDirection = SortDirection.Ascending;
            if (e.SortExpression == lastExpression)
                newDirection = (lastDirection == SortDirection.Ascending) ? SortDirection.Descending : SortDirection.Ascending;

            ViewState["SortExpression"] = e.SortExpression;
            ViewState["SortDirection"] = newDirection;
        }

    }
}
